"""
This package contains the basic core protovis resources to be
used by the widgets packaged in the following:

    - tw2.protovis.conventional
    - tw2.protovis.custom
    - tw2.protovis.interaction
    - tw2.protovis.hierarchies
    - tw2.protovis.networks
    - tw2.protovis.maps
    - tw2.protovis.stats
    - tw2.protovis.art

Get this source from http://github.com/ralphbean/tw2.protovis.core

Protovis itself can be found here:  http://vis.stanford.edu/protovis/
"""

from widgets import *
