<div xmlns:py="http://genshi.edgewall.org/" py:attrs="w.attrs">
<script type="text/javascript">
${w.init_js.src}
var vis = new pv.Panel()
<py:for each="f in w._pv_prop_funcs">
${f.src}
</py:for>

vis.render();

</script>
</div>
